import sys
sys.path.append('..\\')
from keras.models import Sequential
from keras.layers import Dense, Dropout
from os.path import join
from keras_utils.data_io import MatDataGenerator
from ExternalStudents.file_utils import get_rec, make_consecutive_batch, make_pred_array, multiclass_metrics

import numpy as np


import glob
import random
import scipy.io as sio
from sklearn.metrics import confusion_matrix

import pprint

def chunk(xs, n):
    ys = list(xs)
    random.shuffle(ys)
    size = len(ys) // n
    leftovers= ys[size*n:]
    for c in range(n):
        if leftovers:
            extra= [ leftovers.pop() ]
        else:
            extra= []
        yield ys[c*size:(c+1)*size] + extra



window_size = 40
mat_folder = r'C:\temp\EirikLid\f2n'
seed = 0
prefetch_size = 1024

_FAR_LABEL = 'Far'
_NEAR_LABEL = 'Near'
_FAR2NEAR_LABEL = 'Far2Near'
_NEAR2FAR_LABEL = 'Near2Far'

cross_validation_K = 10


input_node_name = 'graph_input'
output_node_name = 'graph_output'





mat_pattern = join(mat_folder, "*.mat")
mat_files = glob.glob(mat_pattern)

if len(mat_files) == 0:
    raise ValueError('Mat file folder {} does not contain any mat files'.format(mat_folder))

np.random.shuffle(mat_files)

mat_chunks = chunk(mat_files,cross_validation_K)
folds = [ fold for fold in mat_chunks]


param_dict = {}
# dropout = np.arange(0.0,1.0,0.1)
# dropout = np.arange(0.1,0.6,0.05)
weight = np.arange(1,29,3)
res = []
for i in range(cross_validation_K):
    print("\nFold nummer :", i)
    test_files = folds[i]
    training_files = [file for fold in (folds[:i]+folds[i+1:]) for file in fold]


    train_datagen_config = {
        'window_size' : window_size,
        'mat_folder': None,
        'mat_files' : training_files,
        'seed' : seed,
        'prefetch_size' : prefetch_size,
        'class_labels' : [_FAR_LABEL, _NEAR_LABEL, _FAR2NEAR_LABEL, _NEAR2FAR_LABEL],
        'class_weight' : [weight[i],weight[i],1,1]
    }

    data_generator_obj = MatDataGenerator.MatDataGenerator(**train_datagen_config)


    def build_dnn_model():
        model = Sequential()
        model.add(Dropout(0.5, input_shape=(data_generator_obj.input_shape,), name=input_node_name))
        # model.add(LSTM(100, input_shape=(data_generator_obj.input_shape), name=input_node_name ))
        # model.add(Flatten())
        # model.add(Dropout(0.25))
        model.add(Dense(64,
                        activation='relu'))

        model.add(Dropout(0.4))

        model.add(Dense(32,
                        activation='relu'))

        model.add(Dropout(0.4))
        model.add(Dense(16,
                        activation='relu'))

        model.add(Dropout(0.4))
        model.add(Dense(4, activation='softmax', name=output_node_name))

        model.compile(loss='categorical_crossentropy',
                      optimizer='Nadam',
                      metrics=['accuracy'])

        return model


    model = build_dnn_model()

    train_stats = model.fit_generator(generator=data_generator_obj.generator(),
                                      steps_per_epoch=5000,
                                      epochs=20,
                                      workers=16,
                                      #validation_data=test_data_generator_obj.generator(),
                                      #validation_steps=4000,
                                      verbose=0)

    # test_datagen_config = {
    #     'window_size' : window_size,
    #     'mat_folder': None,
    #     'mat_files' : test_files,
    #     'seed' : seed,
    #     'prefetch_size' : prefetch_size,
    #     'class_labels' : [_FAR_LABEL, _NEAR_LABEL, _FAR2NEAR_LABEL, _NEAR2FAR_LABEL],
    #     'class_weight' : [19,19,1,1]
    # }
    # test_data_generator_obj = MatDataGenerator.MatDataGenerator(**test_datagen_config)
    #

    print("Train loss and acc")
    print(train_stats.history['loss'])
    print(train_stats.history['acc'])
    test_data = [sio.loadmat(mf, struct_as_record=False, squeeze_me=True) for mf in test_files]
    conf = [[0, 0, 0, 0],
            [0, 0, 0, 0],
            [0, 0, 0, 0],
            [0, 0, 0, 0]]
    for test_seq in test_data:
        batch = make_consecutive_batch.make_flattened(test_seq['X'], window_size=window_size)# num_features=7, stride=stride)
        prediction = model.predict_on_batch(batch)
        offset = test_seq['y'].shape[0] - len(prediction)
        pred = make_pred_array.pred_array(prediction, offset)

        cm = confusion_matrix(test_seq['y'], pred, labels=[0,1,2,3])
        # print(cm)
        conf += cm


    print("Test loss and metric:")
    # print(conf)
    acc = multiclass_metrics.accuracy(conf)
    pre = multiclass_metrics.precision(conf)
    rec = multiclass_metrics.recall(conf)
    print("Acc:", acc, "Precission:", pre, "Recall:", rec)
    res.append(acc)
    param_dict[weight[i]] = acc,pre,rec
scores= np.array(res)
print("Accuracy: %0.2f (+/- %0.2f)" % (scores.mean(), scores.std() * 2))
pprint.pprint(param_dict)


