import scipy.io as sio
import glob
from os import listdir, mkdir
from os.path import join, isdir, isfile, basename
import numpy as np
import random
import scipy



def make_flattened(X, stride=1, num_features=6, window_size=40):

    data_buffer = list()
    feature_sequence_flattened = np.reshape(X, (-1,), order='F')

    batch_stride = stride * num_features
    flattened_window_size = window_size * num_features
    for i in range(0, len(feature_sequence_flattened), batch_stride):
        if i + flattened_window_size > len(feature_sequence_flattened):
            break

        data_buffer.append(feature_sequence_flattened[i:i + flattened_window_size])

    return np.array(data_buffer).astype(np.float32)

def make(X, stride=1, num_features=6, window_size=40):
    data_buffer = list()

    for i in range(0, X.shape[1], stride):
        if i + window_size > X.shape[1]:
            break

        data_buffer.append(X[:num_features, i:i + window_size])

    return np.array(data_buffer)

def make_training_set(files,window_size=20):
    X_list = []
    Y_list = []
    for file in files:
        X_list.append(make(file['X'],window_size=window_size))
        Y_list.append(file['y'][window_size-1:])
    X = np.concatenate(X_list)
    Y = np.concatenate(Y_list)

    return X, Y

def make_flat_set(files):
    X = make_flattened(files[0]['X'])
    Y = files[0]['y']
    i = 0
    for file in files[1:]:
        i += 1
        print(i)
        X = np.concatenate([X, make_flattened(file['X'])], axis=1)
        Y = np.concatenate([Y, file['y']])

    return X, Y

def make_y(y,num_classes):
    batch_size = y.shape[0]
    batch_labels = np.zeros((batch_size, num_classes), dtype=np.float32)

    for i in range(batch_size):
        batch_labels[i,y[i]] = 1.0

    return batch_labels


def chunk(xs, n):
    ys = list(xs)
    random.shuffle(ys)
    size = len(ys) // n
    leftovers= ys[size*n:]
    for c in range(n):
        if leftovers:
            extra= [ leftovers.pop() ]
        else:
            extra= []
        yield ys[c*size:(c+1)*size] + extra

def train_test_split(files, K):
    training_files, test_files = split_files(files,K)

    x_train, y_train = make_training_set(training_files)
    x_test, y_test = make_training_set(test_files)

    return x_train, y_train, x_test, y_test

def split_files(files,K):
    mat_chunks = chunk(files, K)
    folds = [fold for fold in mat_chunks]

    i = 0
    test_files = folds[i]
    training_files = [file for fold in (folds[:i] + folds[i + 1:]) for file in fold]
    return training_files, test_files


def get_len(files,window_size=20):
    file_length = 0
    for file in files:
        file_length += (file['X'].shape[1] - window_size + 1)
    return file_length
