import sys
sys.path.append('..\\')
from keras.models import Sequential
from keras.layers import Dense, Dropout, LSTM, Flatten, GRU
from keras.utils import plot_model, np_utils
from keras.callbacks import TensorBoard
from keras import regularizers
from ExternalStudents.keras_utils.data_io import MatDataGenerator
from ExternalStudents.keras_utils.graph_io import save_optimized_graph

from hyperas import optim
from hyperas.distributions import choice, uniform, conditional
from hyperopt import Trials, STATUS_OK, tpe

from os.path import join
from keras_utils.data_io import MatDataGenerator

import numpy as np
import glob
import random

from ExternalStudents.file_utils import get_rec, make_consecutive_batch


def data():
    # mat_pattern = join(mat_folder, "*.mat")
    # mat_files = glob.glob(mat_pattern)
    #
    # if len(mat_files) == 0:
    #     raise ValueError('Mat file folder {} does not contain any mat files'.format(mat_folder))
    #
    # np.random.shuffle(mat_files)

    files = get_rec.get_recordings(mat_folder =r'C:\temp\EirikLid\us_data')
    x_train, y_train, x_test, y_test = make_consecutive_batch.train_test_split(files,4)
    y_train = np_utils.to_categorical(y_train,4)
    y_test = np_utils.to_categorical(y_test,4)

    return x_train, y_train, x_test, y_test


def create_model(x_train, y_train, x_test, y_test):
    model = Sequential()
    model.add(Dropout({{uniform(0, 1)}}, input_shape=(7,20), name='graph_input'))
    model.add(Flatten())
    model.add(Dense(64,
                    activation={{choice(['relu', 'sigmoid'])}}))

    model.add(Dropout({{uniform(0, 1)}}))

    model.add(Dense(32,
                    activation={{choice(['relu', 'sigmoid'])}}))

    model.add(Dropout({{uniform(0, 1)}}))
    model.add(Dense(16,
                    activation={{choice(['relu', 'sigmoid'])}}))

    model.add(Dense(4, activation='softmax', name='graph_output'))

    model.compile(loss='categorical_crossentropy',
                  optimizer='Nadam',
                  metrics=['accuracy'])

    model.fit(x_train,y_train,
                        epochs=1,
                        # class_weight={0:1,1:1,2:10,3:10},
                        validation_data=(x_test,y_test),
                        verbose=2)

    score, acc = model.evaluate(x_test, y_test,verbose=2)
    print('Test accuracy:', acc)
    return {'loss': -acc, 'status': STATUS_OK, 'model': model}


if __name__ == '__main__':
    best_run, best_model = optim.minimize(model=create_model,
                                          data=data,
                                          algo=tpe.suggest,
                                          max_evals=20,
                                          trials=Trials())

    X_train, Y_train, X_test, Y_test = data()
    # print("Evalutation of best performing model:")
    # print(best_model.evaluate(X_test, Y_test))
    print("Best performing model chosen hyper-parameters:")
    print(best_run)
    best_model.save(r'C:\temp\EirikLid\hypermodel_7_20.h5')


