import sys
sys.path.append('..\\')
from keras.models import Sequential
from keras.layers import Dense, Dropout, LSTM, Flatten, GRU
from keras.utils import plot_model, np_utils
from keras.callbacks import TensorBoard
from keras import regularizers, optimizers
from ExternalStudents.keras_utils.data_io import MatDataGenerator
from ExternalStudents.keras_utils.graph_io import save_optimized_graph

from hyperas import optim
from hyperas.distributions import choice, uniform, conditional, normal, randint
from hyperopt import Trials, STATUS_OK, tpe

from os.path import join
from keras_utils.data_io import MatDataGenerator,SequentialDataGenerator

import numpy as np
import glob
import random

from ExternalStudents.file_utils import get_rec, make_consecutive_batch

import pprint


def data():
    # mat_folder= r'C:\temp\EirikLid\matfiles'
    # mat_pattern = join(mat_folder, "*.mat")
    # mat_files = glob.glob(mat_pattern)
    #
    # if len(mat_files) == 0:
    #     raise ValueError('Mat file folder {} does not contain any mat files'.format(mat_folder))
    #
    # np.random.shuffle(mat_files)
    # train_files, test_files = make_consecutive_batch.split_files(mat_files,4)
    train_datagen_config = {
        'window_size': 20,
        'mat_folder':r'C:\temp\EirikLid\us_data_elliptic',
        'seed': 0,
        'prefetch_size': 1024,
        # 'mat_files': train_files,
        'class_labels': ["Far", "Near", "Far2Near", "Near2Far"],
        'class_weight': [19, 19, 1, 1]
    }

    train_generator = MatDataGenerator.MatDataGenerator(**train_datagen_config, flatten=False)

    test_datagen_config = {
        'window_size': 20,
        'mat_folder':r'C:\temp\EirikLid\us_data_new',
        'seed': 0,
        'prefetch_size': 1024,
        'class_labels': ["Far", "Near", "Far2Near", "Near2Far"],
        'class_weight': [19, 19, 1, 1]
    }

    test_generator = MatDataGenerator.MatDataGenerator(**test_datagen_config, flatten=False)

    return train_generator, test_generator


def create_model(train_generator, test_generator ):
    model = Sequential()
    model.add(Dropout({{uniform(0,1)}}, input_shape=(7,20), name='graph_input'))

    model.add(Flatten())
    model.add(Dense(64,
                    activation={{choice(['relu', 'sigmoid'])}}))

    model.add(Dropout({{uniform(0,1)}}))

    model.add(Dense(32,
                    activation={{choice(['relu', 'sigmoid'])}}))

    model.add(Dropout({{uniform(0,1)}}))
    model.add(Dense(16,
                    activation={{choice(['relu', 'sigmoid'])}}))
    model.add(Dense(4, activation='softmax', name='graph_output'))

    model.compile(loss='categorical_crossentropy',
                  optimizer='nadam',
                  metrics=['accuracy'])

    #model.summary()
    #pprint.pprint(model.get_config())
    model.fit_generator(train_generator.generator(),
                        steps_per_epoch=5000,
                        epochs=10,
                        verbose=2)
    score, acc = model.evaluate_generator(test_generator.generator(),steps=330841//32)
    print('Test accuracy:', acc)
    return {'loss': -acc, 'status': STATUS_OK, 'model': model}


if __name__ == '__main__':
    best_run, best_model= optim.minimize(model=create_model,
                                            data=data,
                                            algo=tpe.suggest,
                                            max_evals=100,
                                            trials=Trials())

    train_generator, test_generator = data()
    print("Evalutation of best performing model:")
    print(best_model.evaluate_generator(test_generator.generator(),steps=330841//32))
    print("Best performing model chosen hyper-parameters:")
    print(best_run)
    best_model.save(r'C:\temp\EirikLid\hypermodel_7_20_us_gen.h5')




