import matplotlib
import matplotlib.pyplot as plt
import numpy as np

import scipy.io as sio
import glob

import matplotlib.collections as collections


def plot_data(file,prediction,stride = 1, us = False, fn_convert=False):
    plt.figure(1)

    t, acc, gyr, y = file['t'][::stride], file['X'][:3,::stride], file['X'][3:6,::stride], file['y'][::stride]

    if(us):
        us_state = file['X'][6,::stride]
    if(fn_convert):
        y = np.where(y == 1, 1, 0) #To remove F2N, N2F predictions
        prediction = np.where(prediction== 1, 1, 0)  # To remove F2N, N2F predictions

    ax1 = plt.subplot(311)

    labels = ["x", "y", "z"]
    for i,line in enumerate(acc):
        plt.plot(t, line, label = labels[i])
    ax1.legend()
    plt.gca().yaxis.set_major_formatter(matplotlib.ticker.FormatStrFormatter(r'%d m/$s^2$'))
    plt.gca().xaxis.set_major_formatter(matplotlib.ticker.FormatStrFormatter(r'%d ms'))
    plt.gca().set_xticks(np.arange(0, max(t), 500), minor=True)

    labels = ["Roll", "Pitch", "Yaw"]
    ax2 = plt.subplot(312, sharex=ax1)

    for i,line in enumerate(gyr):
        plt.plot(t, line, label = labels[i])
    ax2.legend()
    plt.gca().yaxis.set_major_formatter(matplotlib.ticker.FormatStrFormatter('%d rad/s'))


    ax3 = plt.subplot(313, sharex=ax1)

    collection = collections.BrokenBarHCollection.span_where(
        t, ymin=0, ymax=1, where=y == prediction, facecolor='green', alpha=0.5)
    ax3.add_collection(collection)
    collection = collections.BrokenBarHCollection.span_where(
        t, ymin=0, ymax=1, where=y != prediction, facecolor='red', alpha=0.5)
    ax3.add_collection(collection)

    if us:
        plt.plot(t, us_state,'m*', label = "US")
    plt.plot(t, y, label= "IR")
    plt.plot(t, prediction,'k', label = "Prediction")
    ax3.legend()
    y_labels = ["Far", "Near", "Far2Near","Near2Far"]
    plt.yticks(range(4),y_labels)

    ax1.grid( which='both', linestyle='--')
    ax2.grid( which='both', linestyle='--')
    ax3.grid( which='both', linestyle='--')

    ax3.tick_params(which= 'minor', left= 'off')
    plt.subplots_adjust(top=0.99, bottom=0.025, left=0.05, right=0.99, hspace=0.1, wspace=0.2)
    mng = plt.get_current_fig_manager()
    mng.window.showMaximized()
    plt.show()


def compare_state_machine(file,original_prediction, state_machine_pred, stride = 1, us = False, fn_convert=False):
    plt.figure(1)

    t, acc, gyr, y = file['t'][::stride], file['X'][:3,::stride], file['X'][3:6,::stride], file['y'][::stride]

    if(us):
        us_state = file['X'][6,::stride]
    if(fn_convert):
        y = np.where(y == 1, 1, 0) #To remove F2N, N2F predictions
        original_prediction = np.where(original_prediction== 1, 1, 0)  # To remove F2N, N2F predictions

    ax1 = plt.subplot(311)

    labels = ["x", "y", "z"]
    for i,line in enumerate(acc):
        plt.plot(t, line, label = labels[i])
    ax1.legend()
    plt.gca().yaxis.set_major_formatter(matplotlib.ticker.FormatStrFormatter(r'%d m/$s^2$'))
    plt.gca().xaxis.set_major_formatter(matplotlib.ticker.FormatStrFormatter(r'%d ms'))
    plt.gca().set_xticks(np.arange(0, max(t), 500), minor=True)

    labels = ["Roll", "Pitch", "Yaw"]
    ax2 = plt.subplot(312, sharex=ax1)

    for i,line in enumerate(gyr):
        plt.plot(t, line, label = labels[i])
    ax2.legend()
    plt.gca().yaxis.set_major_formatter(matplotlib.ticker.FormatStrFormatter('%d rad/s'))


    ax3 = plt.subplot(313, sharex=ax1)

    collection = collections.BrokenBarHCollection.span_where(
        t, ymin=0, ymax=1, where= np.logical_and(y == original_prediction, y == state_machine_pred), facecolor='green', alpha=0.5)
    ax3.add_collection(collection)

    collection = collections.BrokenBarHCollection.span_where(
        t, ymin=0, ymax=1, where= np.logical_and(y != original_prediction, y == state_machine_pred), facecolor='blue', alpha=0.5)
    ax3.add_collection(collection)

    collection = collections.BrokenBarHCollection.span_where(
        t, ymin=0, ymax=1, where= np.logical_and(y == original_prediction, y != state_machine_pred), facecolor='yellow', alpha=0.5)
    ax3.add_collection(collection)

    collection = collections.BrokenBarHCollection.span_where(
        t, ymin=0, ymax=1, where= np.logical_and(y != original_prediction, y != state_machine_pred), facecolor='red', alpha=0.5)
    ax3.add_collection(collection)


    if us:
        plt.plot(t, us_state,'m*', label = "US")
    plt.plot(t, y, label= "IR")
    plt.plot(t, original_prediction,'k', label = "Original Prediction")
    plt.plot(t, state_machine_pred,'r', label = "State Machine Prediction")
    ax3.legend()
    y_labels = ["Far", "Near", "Far2Near","Near2Far"]
    plt.yticks(range(4),y_labels)

    ax1.grid( which='both', linestyle='--')
    ax2.grid( which='both', linestyle='--')
    ax3.grid( which='both', linestyle='--')

    ax3.tick_params(which= 'minor', left= 'off')
    plt.subplots_adjust(top=0.99, bottom=0.025, left=0.05, right=0.99, hspace=0.1, wspace=0.2)
    mng = plt.get_current_fig_manager()
    mng.window.showMaximized()
    plt.show()
