import sys
sys.path.append('..\\')
from keras.models import Sequential
from keras.layers import Dense, Dropout, LSTM, Flatten, GRU, Lambda
from keras.utils import plot_model
from keras.callbacks import TensorBoard
from keras import regularizers, optimizers
from ExternalStudents.keras_utils.data_io import MatDataGenerator, SequentialDataGenerator
from ExternalStudents.keras_utils.graph_io import save_optimized_graph

import numpy as np

window_size = 40
mat_folder = r'C:\temp\EirikLid\us_data'
seed = 0
prefetch_size = 1024
batch_size = 32
_FAR_LABEL = 'Far'
_NEAR_LABEL = 'Near'
_FAR2NEAR_LABEL = 'Far2Near'
_NEAR2FAR_LABEL = 'Near2Far'


train_datagen_config = {
    'window_size' : window_size,
    'mat_folder' : mat_folder,
    'seed' : seed,
    'prefetch_size' : prefetch_size,
    'class_labels' : [_FAR_LABEL, _NEAR_LABEL, _FAR2NEAR_LABEL, _NEAR2FAR_LABEL],
    'class_weight' : [19,19,1,1],
    # 'data_stride': 4,
    'batch_size': batch_size
}



data_generator_obj = MatDataGenerator.MatDataGenerator(**train_datagen_config,flatten=False)

input_node_name = 'graph_input'
output_node_name = 'graph_output'


def build_dnn_model():
    model = Sequential()
    model.add(Dropout(0.2, input_shape=data_generator_obj.input_shape, name=input_node_name))

    model.add(Dense(64,
                    activation='relu'))

    model.add(Dropout(0.5))

    model.add(Dense(32,
                    activation='relu'))

    model.add(Dropout(0.4))
    model.add(Dense(16,
                    activation='relu'))
    model.add(Dropout(0.5))
    model.add(Dense(4, activation='softmax', name=output_node_name))
    # amsgrad = optimizers.Adam(amsgrad=True)
    model.compile(loss='categorical_crossentropy',
                  optimizer='sgd',
                  metrics=['accuracy'])

    return model


model = build_dnn_model()
model.summary()


train_stats = model.fit_generator(  generator=data_generator_obj.generator(),
                                    steps_per_epoch= 3000,
                                    epochs=2,
                                    workers=16,
                                    verbose=2)


model.save(r'C:\temp\EirikLid\us_MLP_420.h5')


plot_model(model, to_file='model.png', show_shapes=True)